package org.openscm.kundo.strategy

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.plugins.context.BuildContext
import org.openscm.kundo.common.logging.SimpleLogger
import org.openscm.kundo.common.PropertyUtils

 /**
 * Eclipse Project Strategy
 * @author <a href="mailto:ben.leedham@upco.co.uk">Ben Leedham</a>
 * @version 1.0.0
 * 
 * Description: Delegate to load various global properties for use by other plugins
 * and optionally parse eclipse metadata.
 * 
 * <p>The eclipse project strategy reads information from an eclipse .classpath file
 * based on that information creates grouped sets of src and output paths with include 
 * and exclude patterns and writes them to the ant project as a global reference. It 
 * takes the name of the ant project as the global artifactId.</p>
 */
/* 
 * @ant-target doProject
 */
class EclipseProjectStrategy extends AbstractProjectStrategy {
	
	String ps = File.pathSeparator
	def basedir
	def classpathFileLocation
	/**
	 * Constructor sets ant and buildContext instances in super class
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	EclipseProjectStrategy( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext )	
		if( !PropertyUtils.queryProjectForTask( ant, "getSourcepath" ) ){
			ant.'taskdef'( resource:"net/sf/ant4eclipse/antlib.xml", classpathref:"kundo-project-plugin.dependency.path" )
		}		
		basedir = ant.project.getProperty( "basedir" )
		classpathFileLocation = "${basedir}/.classpath"
	}

	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */	
	void doCheck(){
		// assert global variables exists in ant project
		// there are no global variables in this plugin
	}
	
	
	void doProject(){
		if( log.isDebugEnabled() ){
			log.debug( "Entering +EclipseProjectStrategy.doProject()" )
		}
		
		def artifactId = ant.project.getName()
		ant.project.setProperty( "global.artifactid", artifactId )		
		File classpathFile = new File( classpathFileLocation )
		def resolvePaths = ant.project.getProperty( "project.resolve.eclipse.source.paths" )
		
		if( classpathFile.exists() && resolvePaths != "false" ){
			
			log.info( "Attempting to resolve source paths from eclipse metadata" )
			
			ant.getSourcepath(  project:basedir, 
								pathId:"${artifactId}.source.path", 
								property:"${artifactId}.source.property",
								allowMultipleFolders:"true" )
			
			def srcPaths = ant.project.getProperty( "${artifactId}.source.property" )
			Map map = buildContext.get( "project.properties" )
			List srcList = []
			List testList = []
			List resourceList = []
			List testResourceList = []
			
			if( srcPaths != null && srcPaths != "" ){
				srcPaths.split( "${ps}" ).each{ srcPath ->
					
					Expando exp = new Expando()

					ant.getOutputpath(  project:basedir, 
										pathId:"${artifactId}.output.path", 
										property:"${artifactId}.output.property",
										resolve:"forSourceFolder",
										sourceFolder:srcPath.replace( "${basedir}${File.separator}", "" ),
										allowMultipleFolders:"true" )
					
					exp."dir" = srcPath
					def outputProperty = ant.project.getProperty( "${artifactId}.output.property" )
					exp."output" = outputProperty
					def includePattern
					if( srcPath.indexOf( "/" ) != -1 ){
						includePattern = srcPath.split( "/" )
					}else if( srcPath.indexOf( "\\" ) != 1 ){
						if( includePattern == null ){
							includePattern = srcPath.split( "\\\\" )
						}else if( includePattern[ includePattern.length - 1 ].indexOf( "\\" ) ){
							includePattern = includePattern[ includePattern.length - 1 ].split( "\\\\" )
						}
					}
					exp."include" = "**/*.${includePattern[ ( includePattern.length -1 ) ]}"
					exp."exclude" = ""
					
					def relativeSrcPath = srcPath.replace( "${basedir}/", "" ) 
					
					if( relativeSrcPath.indexOf( "main" ) != -1 && relativeSrcPath.indexOf( "resources" ) == -1 ){
						srcList.add( exp )
					}else if( relativeSrcPath.indexOf( "test" ) != -1 && relativeSrcPath.indexOf( "resources" ) == -1 ){
						testList.add( exp )
					}else if( relativeSrcPath.indexOf( "main" ) != -1 && relativeSrcPath.indexOf( "resources" ) != -1 ){
						exp.include = "**/*.*"
						resourceList.add( exp )						
					}else if( relativeSrcPath.indexOf( "test" ) != -1 && relativeSrcPath.indexOf( "resources" ) != -1 ){
						exp.include = "**/*.*"
						testResourceList.add( exp )
					}
				}				
			}else{
				throw new MissingPropertyException( "Unable to find source paths from ${classpathFileLocation}" )
			}
			
			if( !srcList.isEmpty() ){
				map.remove( "src" )
				map.put( "src", srcList )
			}
			
			if( !testList.isEmpty() ){
				map.remove( "test" )
				map.put( "test", testList )				
			}
			
			if( !resourceList.isEmpty() ){
				map.remove( "resources" )
				map.put( "resources", resourceList )				
			}
			
			if( !testResourceList.isEmpty() ){
				map.remove( "testresources" )
				map.put( "testresources", testResourceList )				
			}
			
			ant.project.addReference( "global.project.properties", map )
			
			if( log.isDebugEnabled() ){
				map.each{ key, value ->
					log.debug( "------------------------------" )
					log.debug( "grouped ${key} paths:" )
					value.each{ propObject ->
						log.debug( "	dir: ${propObject.dir}" )
						log.debug( "	output: ${propObject.output}" )
						log.debug( "	include: ${propObject.include}" )
						log.debug( "	exclude: ${propObject.exclude}" )
						log.debug( "	----------------------" )
					}
				}
			}
			
		}else{
			log.debug( "current classpath location is: ${basedir}${File.separator}.classpath and project.eclipse.resolve.source.paths=${resolvePaths}" )
		}
		
		if( log.isDebugEnabled() ){
			log.debug( "Exiting +EclipseProjectStrategy.doProject()" )
		}		
	}
	
	void doMetadata(){
		
	}
	
	/**
	 * Concrete implementation of the AbstractPluginTargetDelegate.execute() method
	 */
	void doExecute(){
		doCheck()
		doProject()
	}
	
}