package org.openscm.kundo.strategy

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import groovy.xml.NamespaceBuilder
import org.openscm.kundo.plugins.context.BuildContext
import org.openscm.kundo.common.logging.SimpleLogger
import org.apache.tools.ant.BuildException

 /**
 * Kundo Project Strategy
 * @author <a href="mailto:ben.leedham@upco.co.uk">Ben Leedham</a>
 * @version 1.0.0
 * 
 * Description: Delegate to load various global properties for use by other plugins
 * 
 * <p>The Kundo Project Strategy will retrieve the Recipe reference set by the kernel
 * from the ant project, it will then retrieve all info that needs to be set as global properties
 * for the other plug-ins in the Kundo suite. These properties are:
 * 
 * global.artifactid       //the current project's artifact Id
 * global.groupid          //the current project's group Id
 * global.artifact.name    //the current project's friendly name
 * global.packaging        //the current project's packaging type</p>
 */
/* 
 * @ant-target doProject
 */
class KundoProjectStrategy extends AbstractProjectStrategy {

	
	/**
	 * Constructor sets ant and buildContext instances in super class
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	KundoProjectStrategy( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
	}

	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */	
	void doCheck(){
		// assert global variables exists in ant project
		// there are no global variables in this plugin
	}	


	void doProject() {
		if( log.isDebugEnabled() ){
			log.debug( "Entering +KundoProjectStrategy.doProject()" )
		}

		//retrieve the recipe reference set by the kernel
		def recipe = buildContext.get( "recipe.hierarchy" )
		
		if( recipe.getArtifact() == null ){
			
			throw new BuildException( "No artifact{} section can be found in your recipe.groovy file. Either specify an artifact{} section or set the project.helper.strategy property to a strategy relevant to your project." )
		}
		
		def artifactId = recipe.getArtifact().getArtifactId()
		def groupId = recipe.getArtifact().getGroupId()
		def packaging = recipe.getArtifact().getPackaging()
		def name = recipe.getArtifact().getName()
		
		ant.property( name : "global.artifactid", value : artifactId )
		ant.property( name : "global.groupid", value : groupId )
		ant.property( name : "global.artifact.name", value : name )
		ant.property( name : "global.packaging", value : packaging )		
		
		if( log.isDebugEnabled() ){
			log.debug( "Exiting +KundoProjectStrategy.doProject()" )
		}		
	}
	
	/**
	 * Concrete implementation of the AbstractPluginTargetDelegate.execute() method
	 */
	void doExecute(){
		doCheck()
		doProject()
	}
	
}