package org.openscm.kundo.strategy

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import groovy.xml.NamespaceBuilder
import org.openscm.kundo.plugins.context.BuildContext
import org.openscm.kundo.common.logging.SimpleLogger
import org.apache.tools.ant.BuildException

 /**
 * Maven Project Strategy
 * @author <a href="mailto:ben.leedham@upco.co.uk">Ben Leedham</a>
 * @version 1.0.0
 * 
 * Description: Delegate to load various global properties for use by other plugins
 * and optionally parse a maven pom file.
 * 
 * <p>The maven project strategy reads a Maven pom file exists the project 
 * artifact information will be taken from it, if not then the project's
 * artifactId will be the name of the ant project. </p>
 */
/* 
 * @ant-target doProject
 */
class MavenProjectStrategy extends AbstractProjectStrategy {

	private def mvn
	
	/**
	 * Constructor sets ant and buildContext instances in super class
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	MavenProjectStrategy( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
		this.mvn = NamespaceBuilder.newInstance( ant, 'antlib:org.apache.maven.artifact.ant' )
	}
	
	/*
	 * The location of the project's pom file if indeed there is one
	 * @ant-property-name project.pom.location
	 */	
	String pomLocation
	
	/**
	 * Setter method for member variable pomLocation
	 * @param pomLocation	String representation the project's pom location
	 */	
	void setPomLocation( String pomLocation ){
		this.pomLocation = pomLocation
	}	
	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */	
	void doCheck(){
		assert pomLocation != null
		// assert global variables exists in ant project
		// there are no global variables in this plugin
	}	


	void doProject() {
		if( log.isDebugEnabled() ){
			log.debug( "Entering +MavenProjectStrategy.doProject()" )
		}

		doPomProperties()
		
		if( log.isDebugEnabled() ){
			log.debug( "Exiting +MavenProjectStrategy.doProject()" )
		}		
	}
	
	/**
	 * reads the pom location variable. If a pom exists then project info is harvested from it
	 * if not then the project artifactId is that of the ant project name
	 *
	 */
	void doPomProperties(){
		if( log.isDebugEnabled() ){
			log.debug( "Entering +MavenProjectStrategy.doPomProperties()" )
		}	
	
		def groupId
		def artifactId
		def version
		def packaging
	
		File pomFile = new File( pomLocation )
		if( pomFile.exists() ){
			if( log.isDebugEnabled() ){
				log.debug( "+MavenProjectStrategy.doPomProperties() pom file exists at location: $pomLocation" )
			}
			
			mvn.pom( file:pomLocation, id:"global.maven.project" )
			groupId = buildContext.get( "maven.project.groupId" )
			artifactId = buildContext.get( "maven.project.artifactId" )
			version = buildContext.get( "maven.project.version" )
			packaging = buildContext.get( "maven.project.packaging" )
			
			ant.project.setProperty( "global.artifactid", artifactId )
			ant.project.setProperty( "global.groupid", groupId )
			ant.project.setProperty( "global.version", version )
			ant.project.setProperty( "global.packaging", packaging )			

		}else{
			throw new BuildException( "pom.xml file does not exist at this location: ${pomLocation} please check your project.pom.location property" )
		}
		
		if( log.isDebugEnabled() ){
			log.debug( "Exiting +MavenProjectStrategy.doPomProperties()" )
		}		
	}
	
	/**
	 * Concrete implementation of the AbstractPluginTargetDelegate.execute() method
	 */
	void doExecute(){
		doCheck()
		doProject()
	}
	
}